@@ -27,12 +27,7 @@
 #include "background.h"
 #include "acquire.h"
 #include "datasources.h"
-#ifdef VIK_CONFIG_GOOGLE
-#include "googlesearch.h"
-#endif
-#ifdef VIK_CONFIG_GEONAMES
-#include "geonamessearch.h"
-#endif
+#include "vikgoto.h"
 #include "dems.h"
 #include "mapcache.h"
 #include "print.h"
@@ -54,6 +49,7 @@
 #include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 
 #define VIKING_WINDOW_WIDTH      1000
 #define VIKING_WINDOW_HEIGHT     800
@@ -307,7 +303,7 @@ static void window_init ( VikWindow *vw )
 
   g_signal_connect_swapped (G_OBJECT(vw->viking_vvp), "expose_event", G_CALLBACK(draw_sync), vw);
   g_signal_connect_swapped (G_OBJECT(vw->viking_vvp), "configure_event", G_CALLBACK(window_configure_event), vw);
-  gtk_widget_add_events ( GTK_WIDGET(vw->viking_vvp), GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK );
+  gtk_widget_add_events ( GTK_WIDGET(vw->viking_vvp), GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK );
   g_signal_connect_swapped (G_OBJECT(vw->viking_vvp), "scroll_event", G_CALLBACK(draw_scroll), vw);
   g_signal_connect_swapped (G_OBJECT(vw->viking_vvp), "button_press_event", G_CALLBACK(draw_click), vw);
   g_signal_connect_swapped (G_OBJECT(vw->viking_vvp), "button_release_event", G_CALLBACK(draw_release), vw);
@@ -510,6 +506,13 @@ static void draw_mouse_motion (VikWindow *vw, GdkEventMotion *event)
   gdouble zoom;
   VikDemInterpol interpol_method;
 
+  /* This is a hack, but work far the best, at least for single pointer systems.
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=587714 for more. */
+  gint x, y;
+  gdk_window_get_pointer (event->window, &x, &y, NULL);
+  event->x = x;
+  event->y = y;
+
   toolbox_move(vw->vt, event);
 
   vik_viewport_screen_to_coord ( vw->viking_vvp, event->x, event->y, &coord );
@@ -535,6 +538,12 @@ static void draw_mouse_motion (VikWindow *vw, GdkEventMotion *event)
   vik_statusbar_set_message ( vw->viking_vs, 4, pointer_buf );
 
   vik_window_pan_move ( vw, event );
+
+  /* This is recommended by the GTK+ documentation, but does not work properly.
+   * Use deprecated way until GTK+ gets a solution for correct motion hint handling:
+   * http://bugzilla.gnome.org/show_bug.cgi?id=587714
+  */
+  /* gdk_event_request_motions ( event ); */
 }
 
 static void vik_window_pan_release ( VikWindow *vw, GdkEventButton *event )
@@ -716,13 +725,30 @@ static void draw_ruler(VikViewport *vvp, GdkDrawable *d, GdkGC *gc, gint x1, gin
     gdk_draw_layout(d, gc, x1-5, y1-CR-3*CW-8, pl);
 
     /* draw label with distance */
-    if (distance >= 1000 && distance < 100000) {
-      g_sprintf(str, "%3.2f km", distance/1000.0);
-    } else if (distance < 1000) {
-      g_sprintf(str, "%d m", (int)distance);
-    } else {
-      g_sprintf(str, "%d km", (int)distance/1000);
+    vik_units_distance_t dist_units = a_vik_get_units_distance ();
+    switch (dist_units) {
+    case VIK_UNITS_DISTANCE_KILOMETRES:
+      if (distance >= 1000 && distance < 100000) {
+	g_sprintf(str, "%3.2f km", distance/1000.0);
+      } else if (distance < 1000) {
+	g_sprintf(str, "%d m", (int)distance);
+      } else {
+	g_sprintf(str, "%d km", (int)distance/1000);
+      }
+      break;
+    case VIK_UNITS_DISTANCE_MILES:
+      if (distance >= 1600 && distance < 160000) {
+	g_sprintf(str, "%3.2f miles", distance/1600.0);
+      } else if (distance < 1600) {
+	g_sprintf(str, "%d yards", (int)(distance*1.0936133));
+      } else {
+	g_sprintf(str, "%d miles", (int)distance/1600);
+      }
+      break;
+    default:
+      g_critical("Houston, we've had a problem. distance=%d", dist_units);
     }
+
     pango_layout_set_text(pl, str, -1);
 
     pango_layout_get_pixel_size ( pl, &wd, &hd );
@@ -800,7 +826,19 @@ static VikLayerToolFuncStatus ruler_click (VikLayer *vl, GdkEventButton *event,
     vik_coord_to_latlon ( &coord, &ll );
     a_coords_latlon_to_string ( &ll, &lat, &lon );
     if ( s->has_oldcoord ) {
-      temp = g_strdup_printf ( "%s %s DIFF %f meters", lat, lon, vik_coord_diff( &coord, &(s->oldcoord) ) );
+      vik_units_distance_t dist_units = a_vik_get_units_distance ();
+      switch (dist_units) {
+      case VIK_UNITS_DISTANCE_KILOMETRES:
+	temp = g_strdup_printf ( "%s %s DIFF %f meters", lat, lon, vik_coord_diff( &coord, &(s->oldcoord) ) );
+	break;
+      case VIK_UNITS_DISTANCE_MILES:
+	temp = g_strdup_printf ( "%s %s DIFF %f miles", lat, lon, vik_coord_diff( &coord, &(s->oldcoord) )* 0.000621371192);
+	break;
+      default:
+	temp = g_strdup_printf ("Just to keep the compiler happy");
+	g_critical("Houston, we've had a problem. distance=%d", dist_units);
+      }
+
       s->has_oldcoord = FALSE;
     }
     else {
@@ -860,7 +898,18 @@ static VikLayerToolFuncStatus ruler_move (VikLayer *vl, GdkEventMotion *event, r
       draw_buf_done = FALSE;
     }
     a_coords_latlon_to_string(&ll, &lat, &lon);
-    temp = g_strdup_printf ( "%s %s DIFF %f meters", lat, lon, vik_coord_diff( &coord, &(s->oldcoord) ) );
+    vik_units_distance_t dist_units = a_vik_get_units_distance ();
+    switch (dist_units) {
+    case VIK_UNITS_DISTANCE_KILOMETRES:
+      temp = g_strdup_printf ( "%s %s DIFF %f meters", lat, lon, vik_coord_diff( &coord, &(s->oldcoord) ) );
+      break;
+    case VIK_UNITS_DISTANCE_MILES:
+      temp = g_strdup_printf ( "%s %s DIFF %f miles", lat, lon, vik_coord_diff( &coord, &(s->oldcoord) )* 0.000621371192);
+      break;
+    default:
+      temp = g_strdup_printf ("Just to keep the compiler happy");
+      g_critical("Houston, we've had a problem. distance=%d", dist_units);
+    }
     vik_statusbar_set_message ( vw->viking_vs, 3, temp );
     g_free ( temp );
   }
@@ -1331,6 +1380,82 @@ GtkWidget *vik_window_get_drawmode_button ( VikWindow *vw, VikViewportDrawMode m
   return mode_button;
 }
 
+static void on_activate_recent_item (GtkRecentChooser *chooser,
+                                     VikWindow *self)
+{
+  gchar *filename;
+
+  filename = gtk_recent_chooser_get_current_uri (chooser);
+  if (filename != NULL)
+  {
+    GFile *file = g_file_new_for_uri ( filename );
+    gchar *path = g_file_get_path ( file );
+    g_object_unref ( file );
+    if ( self->filename )
+    {
+      gchar *filenames[] = { path, NULL };
+      g_signal_emit ( G_OBJECT(self), window_signals[VW_OPENWINDOW_SIGNAL], 0, filenames );
+    }
+    else
+      vik_window_open_file ( self, filename, TRUE );
+    g_free ( path );
+  }
+
+  g_free (filename);
+}
+
+static void setup_recent_files (VikWindow *self)
+{
+  GtkRecentManager *manager;
+  GtkRecentFilter *filter;
+  GtkWidget *menu, *menu_item;
+
+  filter = gtk_recent_filter_new ();
+  /* gtk_recent_filter_add_application (filter, g_get_application_name()); */
+  gtk_recent_filter_add_group(filter, "viking");
+
+  manager = gtk_recent_manager_get_default ();
+  menu = gtk_recent_chooser_menu_new_for_manager (manager);
+  gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (menu), GTK_RECENT_SORT_MRU);
+  gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (menu), filter);
+
+  menu_item = gtk_ui_manager_get_widget (self->uim, "/ui/MainMenu/File/OpenRecentFile");
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
+
+  g_signal_connect (G_OBJECT (menu), "item-activated",
+                    G_CALLBACK (on_activate_recent_item), (gpointer) self);
+}
+
+static void update_recently_used_document(const gchar *filename)
+{
+  /* Update Recently Used Document framework */
+  GtkRecentManager *manager = gtk_recent_manager_get_default();
+  GtkRecentData *recent_data = g_slice_new (GtkRecentData);
+  gchar *groups[] = {"viking", NULL};
+  GFile *file = g_file_new_for_commandline_arg(filename);
+  gchar *uri = g_file_get_uri(file);
+  gchar *basename = g_path_get_basename(filename);
+  g_object_unref(file);
+  file = NULL;
+
+  recent_data->display_name   = basename;
+  recent_data->description    = NULL;
+  recent_data->mime_type      = "text/x-gps-data";
+  recent_data->app_name       = (gchar *) g_get_application_name ();
+  recent_data->app_exec       = g_strjoin (" ", g_get_prgname (), "%f", NULL);
+  recent_data->groups         = groups;
+  recent_data->is_private     = FALSE;
+  if (!gtk_recent_manager_add_full (manager, uri, recent_data))
+  {
+    g_warning (_("Unable to add '%s' to the list of recently used documents"), uri);
+  }
+
+  g_free (uri);
+  g_free (basename);
+  g_free (recent_data->app_exec);
+  g_slice_free (GtkRecentData, recent_data);
+}
+
 void vik_window_open_file ( VikWindow *vw, const gchar *filename, gboolean change_filename )
 {
   switch ( a_file_load ( vik_layers_panel_get_top_layer(vw->viking_vlp), vw->viking_vvp, filename ) )
@@ -1341,6 +1466,7 @@ void vik_window_open_file ( VikWindow *vw, const gchar *filename, gboolean chang
     case 1:
     {
       GtkWidget *mode_button;
+      /* Update UI */
       if ( change_filename )
         window_set_filename ( vw, filename );
       mode_button = vik_window_get_drawmode_button ( vw, vik_viewport_get_drawmode ( vw->viking_vvp ) );
@@ -1358,7 +1484,9 @@ void vik_window_open_file ( VikWindow *vw, const gchar *filename, gboolean chang
       g_assert ( mode_button );
       gtk_check_menu_item_set_active ( GTK_CHECK_MENU_ITEM(mode_button),vik_viewport_get_draw_centermark(vw->viking_vvp) );
     }
-    default: draw_update ( vw );
+    default:
+      update_recently_used_document(filename);
+      draw_update ( vw );
   }
 }
 static void load_file ( GtkAction *a, VikWindow *vw )
@@ -1450,7 +1578,10 @@ static gboolean save_file_as ( GtkAction *a, VikWindow *vw )
 static gboolean window_save ( VikWindow *vw )
 {
   if ( a_file_save ( vik_layers_panel_get_top_layer ( vw->viking_vlp ), vw->viking_vvp, vw->filename ) )
+  {
+    update_recently_used_document ( vw->filename );
     return TRUE;
+  }
   else
   {
     a_dialog_error_msg ( GTK_WINDOW(vw), _("The filename you requested could not be opened for writing.") );
@@ -1488,13 +1619,7 @@ static void acquire_from_gc ( GtkAction *a, VikWindow *vw )
 
 static void goto_address( GtkAction *a, VikWindow *vw)
 {
-#if defined(VIK_CONFIG_GOOGLE) && VIK_CONFIG_SEARCH==VIK_CONFIG_SEARCH_GOOGLE
-  a_google_search(vw, vw->viking_vlp, vw->viking_vvp);
-#endif
-#if defined(VIK_CONFIG_GEONAMES) && VIK_CONFIG_SEARCH==VIK_CONFIG_SEARCH_GEONAMES
-  a_geonames_search(vw, vw->viking_vlp, vw->viking_vvp);
-#endif
-
+  a_vik_goto(vw, vw->viking_vlp, vw->viking_vvp);
 }
 
 static void mapcache_flush_cb ( GtkAction *a, VikWindow *vw )
@@ -1505,6 +1630,7 @@ static void mapcache_flush_cb ( GtkAction *a, VikWindow *vw )
 static void preferences_cb ( GtkAction *a, VikWindow *vw )
 {
   a_preferences_show_window ( GTK_WINDOW(vw) );
+  draw_update ( vw );
 }
 
 static void clear_cb ( GtkAction *a, VikWindow *vw )
@@ -1680,7 +1806,19 @@ static void draw_to_image_file_total_area_cb (GtkSpinButton *spinbutton, gpointe
     w *= gtk_spin_button_get_value(GTK_SPIN_BUTTON(pass_along[4]));
     h *= gtk_spin_button_get_value(GTK_SPIN_BUTTON(pass_along[5]));
   }
-  label_text = g_strdup_printf ( _("Total area: %ldm x %ldm (%.3f sq. km)"), (glong)w, (glong)h, (w*h/1000000));
+  vik_units_distance_t dist_units = a_vik_get_units_distance ();
+  switch (dist_units) {
+  case VIK_UNITS_DISTANCE_KILOMETRES:
+    label_text = g_strdup_printf ( _("Total area: %ldm x %ldm (%.3f sq. km)"), (glong)w, (glong)h, (w*h/1000000));
+    break;
+  case VIK_UNITS_DISTANCE_MILES:
+    label_text = g_strdup_printf ( _("Total area: %ldm x %ldm (%.3f sq. miles)"), (glong)w, (glong)h, (w*h/2589988.11));
+    break;
+  default:
+    label_text = g_strdup_printf ("Just to keep the compiler happy");
+    g_critical("Houston, we've had a problem. distance=%d", dist_units);
+  }
+
   gtk_label_set_text(GTK_LABEL(pass_along[6]), label_text);
   g_free ( label_text );
 }
@@ -1947,6 +2085,7 @@ static GtkActionEntry entries[] = {
 
   { "New",       GTK_STOCK_NEW,          N_("_New"),                          "<control>N", N_("New file"),                                     (GCallback)newwindow_cb          },
   { "Open",      GTK_STOCK_OPEN,         N_("_Open"),                         "<control>O", N_("Open a file"),                                  (GCallback)load_file             },
+  { "OpenRecentFile", NULL,         N_("Open _Recent file"),                         NULL, NULL,                                  (GCallback)NULL             },
   { "Append",    GTK_STOCK_ADD,          N_("A_ppend File"),                  NULL,         N_("Append data from a different file"),            (GCallback)load_file             },
   { "Acquire", NULL, N_("A_cquire"), 0, 0, 0 },
   { "AcquireGPS",   NULL,                N_("From _GPS"),            	  NULL,         N_("Transfer data from a GPS device"),              (GCallback)acquire_from_gps      },
@@ -1966,9 +2105,9 @@ static GtkActionEntry entries[] = {
   { "Exit",      GTK_STOCK_QUIT,         N_("E_xit"),                         "<control>W", N_("Exit the program"),                             (GCallback)window_close          },
   { "SaveExit",  GTK_STOCK_QUIT,         N_("Save and Exit"),                 NULL, N_("Save and Exit the program"),                             (GCallback)save_file_and_exit          },
 
-  { "GotoSearch",   GTK_STOCK_JUMP_TO,                 N_("Go To location"),    	  	  NULL,         N_("Go to address/place using text search"),            (GCallback)goto_address       },
-  { "GotoLL",    GTK_STOCK_QUIT,         N_("_Go to Lat\\/Lon..."),           NULL,         N_("Go to arbitrary lat\\/lon coordinate"),         (GCallback)draw_goto_cb          },
-  { "GotoUTM",   GTK_STOCK_QUIT,         N_("Go to UTM..."),                  NULL,         N_("Go to arbitrary UTM coordinate"),               (GCallback)draw_goto_cb          },
+  { "GotoSearch",   GTK_STOCK_JUMP_TO,   N_("Go To location"),    	  	      NULL,         N_("Go to address/place using text search"),            (GCallback)goto_address       },
+  { "GotoLL",    GTK_STOCK_JUMP_TO,      N_("_Go to Lat\\/Lon..."),           NULL,         N_("Go to arbitrary lat\\/lon coordinate"),         (GCallback)draw_goto_cb          },
+  { "GotoUTM",   GTK_STOCK_JUMP_TO,      N_("Go to UTM..."),                  NULL,         N_("Go to arbitrary UTM coordinate"),               (GCallback)draw_goto_cb          },
   { "SetBGColor",GTK_STOCK_SELECT_COLOR, N_("Set Background Color..."),       NULL,         NULL,                                           (GCallback)set_bg_color          },
   { "ZoomIn",    GTK_STOCK_ZOOM_IN,      N_("Zoom _In"),                   "<control>plus", NULL,                                           (GCallback)draw_zoom_cb          },
   { "ZoomOut",   GTK_STOCK_ZOOM_OUT,     N_("Zoom _Out"),                 "<control>minus", NULL,                                           (GCallback)draw_zoom_cb          },
@@ -2134,6 +2273,8 @@ static void window_create_ui( VikWindow *window )
   accel_group = gtk_ui_manager_get_accel_group (uim);
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
   gtk_ui_manager_ensure_update (uim);
+  
+  setup_recent_files(window);
 }
 
 
